
#define EGL_EGLEXT_PROTOTYPES 1
#define GL_GLEXT_PROTOTYPES 1

#include <cairo.h>
#include <sstream>
#include <string.h>
#include <cairo-xlib.h>
#include <sys/time.h>
#include <cairo-gl.h>
#include <X11/Xlib.h>
#include <EGL/egl.h>
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include <EGL/eglext.h>
#include "window.h"
#include<stdlib.h>
#include<string.h>

int SIZEX = 1280;
int SIZEY = 720;

#define Z_COORD (0)


using namespace std;

char imagename_str[256] = "test.png";
Display *dpy;
Window rootwin;
Window win;
XEvent e;
int scr;
cairo_surface_t *cs = NULL;
EGLDisplay egl_dpy;
EGLContext egl_context;
EGLSurface egl_surface;
cairo_device_t *device_obj;
cairo_surface_t *cairo_image;
cairo_surface_t *glSurface;
cairo_t *gl_cr;
cairo_t *cr;
unsigned char *dri_buffer;
const float M_PI = 3.14f;
EGLConfig       aEGLConfigs[1];
EGLint          cEGLConfigs;

EGLint aEGLContextAttributes[] = {
	EGL_CONTEXT_CLIENT_VERSION, 2,
	EGL_NONE
};

static int cr_fill = 0;
static cairo_operator_t operator_val = CAIRO_OPERATOR_OVER;

#define EGL_CHECK(x) \
	x; \
{ \
	EGLint eglError = eglGetError(); \
	if(eglError != EGL_SUCCESS) { \
	fprintf(stderr, "eglGetError() = %i (0x%.8x) at line %i\n", eglError, eglError, __LINE__); \
	exit(1); \
	} \
}


void InitEGL( Display *dpy, Window awin)
{
	EGLint aEGLAttributes[] = {
		EGL_SURFACE_TYPE, EGL_WINDOW_BIT | EGL_PBUFFER_BIT | EGL_PIXMAP_BIT,
		EGL_BUFFER_SIZE, 32,
		EGL_RED_SIZE, 8,
		EGL_GREEN_SIZE, 8,
		EGL_BLUE_SIZE, 8,
		EGL_ALPHA_SIZE, 8,
		EGL_DEPTH_SIZE, 16,
		EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
		EGL_NONE
	};

	egl_dpy = EGL_CHECK(eglGetDisplay(dpy));

	int major, minor;
	EGL_CHECK(eglInitialize(egl_dpy, &major, &minor));
	printf("Initialzed Version = %d.%d\n\n", major, minor );

	EGL_CHECK(eglChooseConfig(egl_dpy, aEGLAttributes, aEGLConfigs, 1, &cEGLConfigs));

	if (cEGLConfigs == 0) {
		printf("No EGL configurations were returned.\n");
		exit(-1);
	}

	Colormap colormap;
	XVisualInfo *pVisual;	
	win = create_window("OpenGL ES 2.0 DRI", SIZEX, SIZEY, dpy, egl_dpy, aEGLConfigs[0], &colormap, &pVisual);	

	egl_surface = EGL_CHECK(eglCreateWindowSurface(egl_dpy, aEGLConfigs[0], (EGLNativeWindowType)win, NULL));

	if (egl_surface == EGL_NO_SURFACE)
	{
		fprintf(stderr, "Failed to create EGL surface.\n");
		exit(-1);
	}
	egl_context = EGL_CHECK(eglCreateContext(egl_dpy, aEGLConfigs[0], EGL_NO_CONTEXT, aEGLContextAttributes));
	if (egl_context == EGL_NO_CONTEXT)
	{
		printf("Failed to create EGL context.\n");
		exit(-1);
	}


}


/* return current time (in seconds) */
static double current_time(void)
{
	struct timeval tv;
	struct timezone tz;
	(void) gettimeofday(&tv, &tz);
	return (double) tv.tv_sec + tv.tv_usec / 1000000.0;
}


void DrawTest(cairo_t *cr, cairo_surface_t *render_surface)
{
	static int x = 0;
	static float rotate_angle = 0.0;
	cairo_save(cr);

	cairo_translate(cr, cairo_image_surface_get_width(cairo_image)/2, cairo_image_surface_get_height(cairo_image)/2);
	cairo_rotate(cr, rotate_angle);
	cairo_translate(cr, -cairo_image_surface_get_width(cairo_image)/2, -cairo_image_surface_get_height(cairo_image)/2);
	rotate_angle = rotate_angle + 0.05;

	if(rotate_angle >= 360)
	{
		rotate_angle = 0.0;
	}

	/*Below code to paint gray background*/
	cairo_set_operator(cr, operator_val);
	cairo_set_source_rgb(cr, 0.25, 0.25, 0.25);
	cairo_paint(cr);

	if(cr_fill)
	{
		cairo_rectangle(cr, 0, 0, cairo_image_surface_get_width(cairo_image), cairo_image_surface_get_height(cairo_image));
		cairo_set_source_surface(cr, render_surface, 0, 0);
		cairo_fill(cr);
	}
	else
	{
		//Below code gives gl error 1282 and distorted output is seen for SOURCE OPERATOR. Works fine for OVER OPERATOR.
		cairo_rectangle(cr, 0, 0, cairo_image_surface_get_width(cairo_image), cairo_image_surface_get_height(cairo_image));
		cairo_clip(cr);
		cairo_set_source_surface(cr, render_surface, 0, 0);
		cairo_paint(cr);
	}

	cairo_restore(cr);

}

void ProcessArguments(int argc, char *argv[])
{
	for (int i = 1; i < argc; i++)
        {
                if (strcmp(argv[i], "-720p") == 0)
                {
                        SIZEX = 1280; 
                        SIZEY = 720;
                }
                else if (strcmp(argv[i], "-1080p") == 0)
                {
                        SIZEX = 1920;
                        SIZEY = 1080;
                }
                else if (strcmp(argv[i], "-540p") == 0)
                {
                        SIZEX = 960;
                        SIZEY = 540;
                }
                else if (strcmp(argv[i], "-cairo_fill") == 0)
                {
			cr_fill = 1;
                }
                else if (strcmp(argv[i], "-cairo_clip") == 0)
                {
			cr_fill = 0;
                }
                else if (strcmp(argv[i], "-rop_source") == 0)
                {
			operator_val = CAIRO_OPERATOR_SOURCE;  //Cairo gl or image rendering with ROP= Source
                }
                else if (strcmp(argv[i], "-rop_over") == 0)
                {
			operator_val = CAIRO_OPERATOR_OVER; //Cairo gl ir image rendering with ROP= OVER
                }
                else if (strcmp(argv[i], "-image_name") == 0)
                {
			if(argv[i+1] != NULL) 
			{
				sprintf(imagename_str, "%s", argv[++i]); // PNG Image to render with path. Default is test.png
			}
			else
			{
				fprintf(stderr, "*** WARNING : provide image name. ie -image_name test.png \n");
			}
                }
                else
                {
			fprintf(stderr, "WARNING! Not a Valid Argument \n");
                }
        }
}


void LoadImage(char* filename)
{
	//TODO:  BPM image loading
	cairo_image = cairo_image_surface_create_from_png(filename);
}


void ProcessSurfaceType()
{
	LoadImage(imagename_str);

	fprintf(stderr, "************** SURFACE TYPE EGL ************* \n");
	device_obj = cairo_egl_device_create(egl_dpy, egl_context);
	cairo_gl_device_set_thread_aware(device_obj, 0);
	cs = cairo_gl_surface_create_for_egl(device_obj, egl_surface, SIZEX, SIZEY);
	glSurface = cairo_gl_surface_create(device_obj, CAIRO_CONTENT_COLOR_ALPHA, SIZEX, SIZEY);
	gl_cr = cairo_create(glSurface);
	cairo_set_source_surface(gl_cr, cairo_image, 0, 0);
	cairo_paint(gl_cr);
	cr      = cairo_create(cs);

}


int main(int argc, char *argv[])
{

	ProcessArguments(argc, argv);
	if(!(dpy=XOpenDisplay(NULL))) {
		fprintf(stderr, "ERROR: Could not open display\n");
		exit(1);
	}
	InitEGL(dpy, win);	
	ProcessSurfaceType();

	while(1) {

		static int frames = 0;
		static double tRot0 = -1.0, tRate0 = -1.0;
		double dt, t = current_time();	

		DrawTest(cr, glSurface);
		cairo_gl_surface_swapbuffers(cs);


		//TO SHOW FPS
		frames++;
		if (tRate0 < 0.0)
		{
			tRate0 = t;
		}
		if (t - tRate0 >= 2.0) {
			GLfloat seconds = t - tRate0;
			GLfloat fps = frames / seconds;
			printf("%d frames in %3.1f seconds = %6.3f FPS\n", frames, seconds,
				fps);
			tRate0 = t;
			frames = 0;
		}
	}

	if(cs != NULL)
	{
		cairo_surface_destroy(cs);
	}
	XCloseDisplay(dpy);

	return 0;
}

