#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include "window.h"


/* Waits for map notification */
Bool wait_for_map(Display *d, XEvent *e, char *win_ptr) {
	return (e->type == MapNotify && e->xmap.window == (*((Window*)win_ptr)));
}

/* Creates an X window */
Window create_window(const char *title, int width, int height, Display* display,
					 EGLDisplay sEGLDisplay, EGLConfig FBConfig, Colormap *pColormap,
					 XVisualInfo **ppVisual)
{
	XSetWindowAttributes wa;
	XSizeHints sh;
	XEvent e;
	XVisualInfo *visual, templat;

	unsigned long mask;
	long screen;

	Colormap colormap;

	int vID, n;

	Window window;

	screen = DefaultScreen(display);
	eglGetConfigAttrib(sEGLDisplay, FBConfig, EGL_NATIVE_VISUAL_ID, &vID);
	templat.visualid = vID;
	visual = XGetVisualInfo(display, VisualIDMask, &templat, &n);

	if (!visual) {
		printf("Couldn't get X visual info\n");
#ifndef X86
		exit(-1);
#endif
	}

#ifndef X86
	colormap = XCreateColormap(display, RootWindow(display, screen), visual->visual, AllocNone);

	wa.colormap = colormap;
#endif
	wa.background_pixel = 0xFFFFFFFF;
	wa.border_pixel = 0;
	wa.event_mask = StructureNotifyMask | ExposureMask;
	wa.override_redirect = True;

	mask = CWBackPixel | CWBorderPixel | CWEventMask
#ifndef X86
		| CWColormap
#endif
		;


	window = XCreateWindow(display, RootWindow(display, screen), 0, 0, width, height,
#ifdef X86
		0, visual->depth, InputOutput, visual->visual, mask, &wa);
#else
		0, CopyFromParent, InputOutput, CopyFromParent, CWEventMask, &wa );
#endif

	sh.flags = USPosition;
	sh.x = 10;
	sh.y = 10;

	XSetStandardProperties(display, window, title, title, None, 0, 0, &sh);
	XMapWindow(display, window);


	XIfEvent(display, &e, wait_for_map, (char*)&window);
	XSetWMColormapWindows(display, window, &window, 1);
	XFlush(display);

	*pColormap = colormap;
	*ppVisual = visual;

	return window;
}



